#pragma once

#include <vector>
#include <map>
#include "oclub.h"

const int baseNameLength=32;

//Has 0-clearing constructor. Must not contain any
//dynamic data etc.
struct RunnerDBEntry {
  RunnerDBEntry();

  char name[baseNameLength];
  int cardNo;
  int clubNo;
  char national[3];
  char sex;
  short int birthYear;
  short int reserved;

  void getName(string &name) const;
  void setName(const char *name);

  string getGivenName() const;
  string getFamilyName() const;
};

typedef vector<RunnerDBEntry> RunnerDBVector;

class RunnerDB {
private:
  oEvent *oe;
  bool check(const RunnerDBEntry &rde) const;

  vector<RunnerDBEntry> rdb;
  vector<oClub> cdb;
  map<int, int> rhash;
  map<int, int> chash;
public:
  void addClub(oClub &c);
  RunnerDBEntry *addRunner(const char *name, int club, int card);
  bool getRunner(int card, string &name, string &club) const;
  RunnerDBEntry *RunnerDB::getRunner(int card) const;
  bool getClub(int clubId, string &club) const;
  oClub *getClub(int clubId) const;
  
  oClub *RunnerDB::getClub(const string &name) const; 
  
  void addCR(int card, const string &name, const string &club);                
  void saveClubs(const char *file);
  void saveRunners(const char *file);
  void loadRunners(const char *file);
  void loadClubs(const char *file);
  
  void updateAdd(int card, const string &name, const pClub club);

  void importClub(oClub &club);
  void compactifyClubs();

  void getAllNames(vector<string> &givenName, vector<string> &familyName);
  RunnerDB(oEvent *);
  ~RunnerDB(void);
};
