/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

#include "stdafx.h"

#include "resource.h"

#include <commctrl.h>
#include <commdlg.h> 

#include "oEvent.h"
#include "xmlparser.h"
#include "gdioutput.h"
#include "csvparser.h"
#include "SportIdent.h"
#include "meos_util.h"
#include "TabCompetition.h"
#include "oFreeImport.h"
#include "localizer.h"

#include <cassert>
#include <cmath>

void Setup(bool overwrite);
void exportSetup();

TabCompetition::TabCompetition(oEvent *poe):TabBase(poe)
{
  assert(!FixedTabs::cmpTab);
  FixedTabs::cmpTab=this;
  defaultServer="localhost";
  defaultName="meos";
}

TabCompetition::~TabCompetition(void)
{
  FixedTabs::cmpTab=0;
}

extern SportIdent *gSI;
extern HINSTANCE hInst;
extern HWND hWndMain;

bool TabCompetition::save(gdioutput &gdi, bool write)
{
	string name=gdi.getText("Name");

  if(name.empty()) {
		gdi.alert("Tvlingen mste ha ett namn");
		return 0;
	}

	oe->setName(gdi.getText("Name"));
	oe->setDate(gdi.getText("Date"));
	oe->setZeroTime(gdi.getText("ZeroTime"));
  
  oe->syncronize();
	if(gSI) gSI->SetZeroTime(oe->getZeroTimeNum());

	gdi.setWindowTitle(oe->getTitleName());

  if (write) {
	  gdi.setWaitCursor(true);
	  return oe->save();
  }
  else
    return true;
}

string TabCompetition::browseForOpen(const string &Filter, const string &defext)
{
	InitCommonControls();
	
	char FileName[260];
	FileName[0]=0;
	char sbuff[256];
	OPENFILENAME of;
	sprintf_s(sbuff, 256, "%s", Filter.c_str());

	int sl=strlen(sbuff);
	for(int m=0;m<sl;m++) if(sbuff[m]=='') sbuff[m]=0;

	of.lStructSize      =sizeof(of);
	of.hwndOwner        =hWndMain;
	of.hInstance        =hInst;
	of.lpstrFilter      =sbuff;
	of.lpstrCustomFilter=NULL;
	of.nMaxCustFilter   =0;
	of.nFilterIndex     =1;
	of.lpstrFile        = FileName;
	of.nMaxFile         =260;
	of.lpstrFileTitle   =NULL;
	of.nMaxFileTitle    =0;
	of.lpstrInitialDir  =NULL;
	of.lpstrTitle       =NULL;
	of.Flags            =OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST|OFN_HIDEREADONLY;
	of.lpstrDefExt   	  = defext.c_str();
	of.lpfnHook		     =NULL;
	
	
	if(GetOpenFileName(&of)==false)
		return "";
	
	return FileName;
}

string TabCompetition::browseForSave(const string &Filter, const string &defext, int &FilterIndex)
{
	InitCommonControls();
	
	char FileName[260];
	FileName[0]=0;
	char sbuff[256];
	OPENFILENAME of;
//	LoadString(hInst,1016,sbuff,256);
	sprintf_s(sbuff, 256,"%s", Filter.c_str());

	int sl=strlen(sbuff);
	for(int m=0;m<sl;m++) if(sbuff[m]=='') sbuff[m]=0;

	of.lStructSize      = sizeof(of);
	of.hwndOwner        = hWndMain;
	of.hInstance        = hInst;
	of.lpstrFilter      = sbuff;
	of.lpstrCustomFilter= NULL;
	of.nMaxCustFilter   = 0;
	of.nFilterIndex     = 1;
	of.lpstrFile        = FileName;
	of.nMaxFile         = 260;
	of.lpstrFileTitle   = NULL;
	of.nMaxFileTitle    = 0;
	of.lpstrInitialDir  = NULL;
	of.lpstrTitle       = NULL;
	of.Flags            = OFN_OVERWRITEPROMPT|OFN_HIDEREADONLY;
	of.lpstrDefExt   	  = defext.c_str();
	of.lpfnHook		      = NULL;
	
	if(GetSaveFileName(&of)==false)
		return "";

	FilterIndex=of.nFilterIndex;

	return FileName;
}

bool TabCompetition::importFile(HWND hWnd, gdioutput &gdi)
{
	InitCommonControls();
	
	char FileName[260];
	FileName[0]=0;
	char sbuff[256];
	OPENFILENAME of;
//	LoadString(hInst,1016,sbuff,256);
	sprintf_s(sbuff, 256, "XML-data*.xml;*.bu?");

	int sl=strlen(sbuff);
	for(int m=0;m<sl;m++) if(sbuff[m]=='') sbuff[m]=0;

	of.lStructSize      =sizeof of;
	of.hwndOwner        =hWnd;
	of.hInstance        =hInst;
	of.lpstrFilter      =sbuff;
	of.lpstrCustomFilter=NULL;
	of.nMaxCustFilter   =0;
	of.nFilterIndex     =1;
	of.lpstrFile        = FileName;
	of.nMaxFile         =260;
	of.lpstrFileTitle   =NULL;
	of.nMaxFileTitle    =0;
	of.lpstrInitialDir  =NULL;
	of.lpstrTitle       =NULL;
	of.Flags            =OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST|OFN_HIDEREADONLY;
	of.lpstrDefExt   	  ="xml";
	of.lpfnHook		     =NULL;
	
	
	if(GetOpenFileName(&of)==false)
		return false;
	
	gdi.setWaitCursor(true);
	if(oe->open(FileName, true)) {
		if(gSI) gSI->SetZeroTime(oe->getZeroTimeNum());
		gdi.setWindowTitle(oe->getTitleName());
		return true;
	}

	return false;
}

bool TabCompetition::exportFileAs(HWND hWnd, gdioutput &gdi)
{
	InitCommonControls();
	
	char FileName[260];
	FileName[0]=0;
	char sbuff[256];

	OPENFILENAME of;
	//LoadString(hInst,1016,sbuff,256);
	sprintf_s(sbuff, 256, "XML-data*.xml");
	
	int sl=strlen(sbuff);
	for(int m=0;m<sl;m++) if(sbuff[m]=='') sbuff[m]=0;

	of.lStructSize      =sizeof of;
	of.hwndOwner        =hWnd;
	of.hInstance        =hInst;
	of.lpstrFilter      =sbuff;
	of.lpstrCustomFilter=NULL;
	of.nMaxCustFilter   =0;
	of.nFilterIndex     =1;
	of.lpstrFile        = FileName;
	of.nMaxFile         = 260;
	of.lpstrFileTitle   = NULL;
	of.nMaxFileTitle    = 0;
	of.lpstrInitialDir  = NULL;
	of.lpstrTitle       = NULL;
	of.Flags            = OFN_OVERWRITEPROMPT|OFN_HIDEREADONLY;
	of.lpstrDefExt   	  = "xml";
	of.lpfnHook		      = NULL;
	
	
	if(GetSaveFileName(&of)==false)
		return false;

	gdi.setWaitCursor(true);
	if(!oe->save(FileName)) {
		gdi.alert("Fel: Filen "+string(FileName)+ " kunde inte skrivas.");
		return false;
	}

	return true;
}

int CompetitionCB(gdioutput *gdi, int type, void *data)
{
  if (!FixedTabs::cmpTab) 
		throw std::exception("tabCmp undefined.");

  return FixedTabs::cmpTab->competitionCB(*gdi, type, data);
}

void TabCompetition::loadConnectionPage(gdioutput &gdi)
{
  gdi.clearPage(false);
  showConnectionPage=true;    
  gdi.addString("", boldLarge, "Anslutningar");

  if(oe->getServerName().empty()) {
    gdi.addString("", 10, "help:52726");
    gdi.pushX();
    
    gdi.fillRight();
	  gdi.addInput("Server", defaultServer, 16, 0, "MySQL Server / IP-adress:", "IP-adress eller namn p en MySQL-server");
	  gdi.addInput("UserName", defaultName, 7, 0, "Anvndarnamn:");
	  gdi.addInput("PassWord", defaultPwd, 9, 0, "Lsenord:");
	  gdi.addInput("Port", defaultPort, 4, 0, "Port:");
    gdi.fillDown();
    gdi.popX();
    gdi.dropLine(2.5);
    gdi.addInput("ClientName", oe->getClientName(), 16, 0, "Klientnamn");
    gdi.dropLine();
    gdi.fillRight();
    gdi.addButton("ConnectToMySQL", "Anslut", CompetitionCB);
  }
  else {
    gdi.addString("", 10, "help:50431");
    gdi.dropLine(1);
    gdi.pushX();
    gdi.fillRight();
    gdi.addString("", 1, "Ansluten till:");
    gdi.addString("", 1, oe->getServerName()).setColor(200,0,0);
    gdi.popX();
    gdi.dropLine(2);
    gdi.addInput("ClientName", oe->getClientName(), 16, 0, "Klientnamn:");
    gdi.dropLine();
    gdi.addButton("SaveClient", "ndra", CompetitionCB);
    gdi.dropLine(2.5);

    gdi.popX();
    gdi.addString("", 1, "ppnad tvling:");
    
    if(oe->empty())
      gdi.addString("", 1, "Ingen").setColor(200,0,0);
    else {
      gdi.addString("", 1, oe->getName()).setColor(200,0,0);
    
      if(oe->isClient())
        gdi.addString("", 1, "(p server)");
      else
        gdi.addString("", 1, "(lokalt)");

    }
    gdi.dropLine(2);
    gdi.popX();
    gdi.fillRight();

    if(!oe->isClient())
      gdi.addButton("UploadCmp", "Ladda upp ppnad tvling p server",CompetitionCB);
   
    if(oe->empty())
      gdi.disableInput("UploadCmp");
    else {
      gdi.addButton("CloseCmp", "Stng tvlingen", CompetitionCB);
      gdi.addButton("Delete", "Radera tvlingen", CompetitionCB);
    }
    gdi.dropLine(2);
    gdi.popX();
    if (oe->empty()) {
		  char bf[260];
		  GetUserFile(bf, "");
		  oe->enumerateCompetitions(bf, "*.meos");		
  		
      gdi.dropLine(1);
      gdi.fillRight();
      gdi.addListBox("ServerCmp", 320, 210,  CompetitionCB, "Server");
      oe->fillCompetitions(gdi, "ServerCmp", 2); 
      gdi.fillDown();   
      gdi.addListBox("LocalCmp", 320, 210, CompetitionCB, "Lokalt"); 
      gdi.popX();
      oe->fillCompetitions(gdi, "LocalCmp", 1);

      gdi.fillRight();
      gdi.addButton("OpenCmp", "ppna tvling", CompetitionCB);    
    }
    else if (oe->isClient()) {
      gdi.fillDown();
      gdi.popX();
      oe->listConnectedClients(gdi);
      gdi.registerEvent("Connections", CompetitionCB);
      gdi.fillRight();
    }

    gdi.addButton("DisconnectMySQL", "Koppla ner databas", CompetitionCB);
  }
  gdi.addButton("Cancel", "Till huvudsidan", CompetitionCB);
  gdi.fillDown();
  gdi.refresh();
}

int TabCompetition::competitionCB(gdioutput &gdi, int type, void *data)
{
	if (type==GUI_BUTTON) {
		ButtonInfo bi=*(ButtonInfo *)data;

    if (bi.id=="UseEconomy" || bi.id=="UseSpeaker") {
      oe->GetEventDI().setInt("UseEconomy", gdi.isChecked("UseEconomy"));
      oe->GetEventDI().setInt("UseSpeaker", gdi.isChecked("UseSpeaker"));
      oe->updateTabs();
    }
    if (bi.id=="Print")	{
			gdi.print(oe);			
		}
    else if (bi.id=="Setup") {
      gdi.clearPage(false);

      gdi.addString("", boldLarge, "Instllningar MeOS");
      gdi.dropLine();
      gdi.addString("", 10, "Du kan installera instllningar, klubb- och lpardatabaser "
                            "frn en kllkatalog som du anger. De lokala instllningarna "
                            "skrivs ver. Efter installationen kan MeOS behva startas om."
                            "\n\nKnappen <Exportera> tar dig till en sida dr du istllet "
                            "kan exportera dina nuvarande lokala instllningar.");
      gdi.dropLine();
      char FileNamePath[260];
      GetUserFile(FileNamePath, "");
      gdi.addStringUT(0, lang.tl("MeOS lokala datakatalog r") + ": " + FileNamePath);
      gdi.dropLine();

      char bf[260];
      GetCurrentDirectory(260, bf);
      gdi.fillRight();
      gdi.pushX();
      gdi.dropLine();
      gdi.addInput("Source", bf, 50, 0, "Kllkatalog:");
      gdi.dropLine(0.3);
      gdi.addButton("SourceBrowse", "Blddra...", CompetitionCB); 
      gdi.dropLine(4);
      gdi.popX();
      gdi.fillRight();
      gdi.addButton("DoSetup", "Installera", CompetitionCB);
      gdi.addButton("ExportSetup", "Exportera", CompetitionCB);
      gdi.addButton("Cancel", "Avbryt", CompetitionCB);
      gdi.dropLine(2);
      gdi.refresh();
    }
    else if (bi.id=="SourceBrowse") {
      string s = gdi.browseForFolder(gdi.getText("Source"));
      if (!s.empty())
        gdi.setText("Source", s);
    }
    else if (bi.id=="DoSetup") {
      string source = gdi.getText("Source");
      if(SetCurrentDirectory(source.c_str())) {
        Setup(true);
        gdi.alert("Tillgngliga filer installerades. Starta om MeOS.");
        exit(0);
      }
      else
        throw std::exception("Operationen misslyckades");
    }
    else if (bi.id=="ExportSetup") {
      gdi.clearPage(false);

      gdi.addString("", boldLarge, "Exportera instllningar och lpardatabaser");
      gdi.addString("", 10, "help:15491");
      gdi.dropLine();
      char FileNamePath[260];
      GetUserFile(FileNamePath, "");
      gdi.addStringUT(0, lang.tl("MeOS lokala datakatalog r") + ": " + FileNamePath);
      
      gdi.dropLine();

      char bf[260];
      GetCurrentDirectory(260, bf);
      gdi.fillRight();
      gdi.pushX();
      gdi.dropLine();
      gdi.addInput("Source", bf, 50, 0, "Destinationskatalog:");
      gdi.dropLine(0.3);
      gdi.addButton("SourceBrowse", "Blddra...", CompetitionCB); 
      gdi.dropLine(4);
      gdi.popX();
      gdi.fillRight();
      gdi.addButton("DoExportSetup", "Exportera", CompetitionCB);
      gdi.addButton("Cancel", "Avbryt", CompetitionCB);
      gdi.dropLine(2);
      gdi.refresh();
    }
    else if (bi.id=="DoExportSetup") {
      string source = gdi.getText("Source");
      if(SetCurrentDirectory(source.c_str())) {
        exportSetup();
        gdi.alert("Instllningarna har exporterats.");
      }
    }
    else if (bi.id=="Report") {
      gdi.clearPage(true);
      oe->generateCompetitionReport(gdi);

      gdi.addButton(gdi.getWidth()+20, 15, "Cancel", "terg", CompetitionCB, "", "", true);
      gdi.addButton(gdi.getWidth()+20, 45, "Print", "Skriv ut...", CompetitionCB, "", "Skriv ut rapporten.", true);
      gdi.refresh();

      //gdi.addButton("Cancel", "Avbryt", CompetitionCB);    
    }
    else if (bi.id=="Settings") {
      gdi.clearPage(false);

      gdi.addString("", boldText, "Tvlingsinstllningar");
      oe->GetEventDI().buildDataFields(gdi);
      oe->GetEventDI().fillDataFields(gdi);

      gdi.fillRight();
      gdi.addButton("SaveSettings", "Spara", CompetitionCB);
      gdi.addButton("Cancel", "Avbryt", CompetitionCB);
      gdi.dropLine(2);
      gdi.refresh();
    }
    else if(bi.id=="SaveSettings") {
      oe->GetEventDI().saveDataFields(gdi);
      loadPage(gdi);
    }
    else if (bi.id=="Browse") {
      string f = gdi.browseForOpen("XML-filer*.xml", "xml");
      string id;
      InputInfo *ii;
      if (!f.empty()) {
        ii = static_cast<InputInfo *>(bi.getExtra());
        if (ii)
          gdi.setText(ii->id, f);
      }
    }
    else if (bi.id=="DBaseIn") {
      gdi.clearPage(true);
      gdi.addString("", boldText, "Importera lpare och klubbar / distriktsregister");
      gdi.addString("", 10, "help:687188");
      gdi.dropLine(2);
      gdi.pushX();
      gdi.fillRight();
      InputInfo *ii = &gdi.addInput("ClubFile", "", 20, 0, "Filnamn IOF-XML med klubbar");
      gdi.dropLine();
      gdi.addButton("Browse", "Blddra...", CompetitionCB).setExtra(ii);
      gdi.popX();
      gdi.dropLine(3);
      ii = &gdi.addInput("CmpFile", "", 20, 0, "Filnamn IOF-XML med lpare");
      gdi.dropLine();
      gdi.addButton("Browse", "Blddra...", CompetitionCB).setExtra(ii);
      
      gdi.dropLine(4);
      gdi.popX();
      gdi.addButton("DoDBaseIn", "Importera", CompetitionCB);
      gdi.addButton("Cancel", "Avbryt", CompetitionCB);
      gdi.dropLine(3);
      gdi.fillDown();
      gdi.popX();
    }
    else if (bi.id=="DoDBaseIn") {
      gdi.setWaitCursor(true);
      gdi.addString("", 0, "Importerar...");
      oe->importXML_IOF_Data(gdi.getText("ClubFile").c_str(),
                             gdi.getText("CmpFile").c_str());
      gdi.addString("", 0, "OK");
      gdi.setWaitCursor(false);                       
    }
    else if (bi.id=="Reset") {
      if (gdi.ask("Vill d terstlla instllningar och skriva ver egna databaser?"))
        Setup(true);
    }
		else if (bi.id=="ConnectMySQL") 
      loadConnectionPage(gdi);
    else if(bi.id=="SaveClient") 
      oe->setClientName(gdi.getText("ClientName"));
		else if (bi.id=="ConnectToMySQL") {
			bool s=oe->connectToMySQL(gdi.getText("Server"), 
                                gdi.getText("UserName"),
                                gdi.getText("PassWord"), 
                                gdi.getTextNo("Port"));

      if (s) {
        defaultServer=gdi.getText("Server");
        defaultName=gdi.getText("UserName");
        defaultPwd=gdi.getText("PassWord");
        defaultPort=gdi.getText("Port");

        oe->setClientName(gdi.getText("ClientName"));
        loadConnectionPage(gdi);
      }
		}
    else if (bi.id=="DisconnectMySQL") {
      oe->closeDBConnection();
      loadConnectionPage(gdi);
		}
    else if (bi.id=="UploadCmp") {
      if(oe->uploadSyncronize())
        gdi.setWindowTitle(oe->getTitleName());

			loadConnectionPage(gdi);
    }
		else if(bi.id=="Cancel"){
			loadPage(gdi);
		}
    else if(bi.id=="dbtest") {
      
    }
    else if(bi.id=="FreeImport") {
      gdi.clearPage(true);
      gdi.addString("", 2, "Fri anmlningsimport");
      gdi.addString("", 10, "help:33940");
      gdi.dropLine(0.5);
      gdi.addInputBox("EntryText", 450, 280, entryText, 0, "");
      gdi.dropLine(0.5);
      gdi.fillRight();
      gdi.addButton("PreviewImport", "Granska inmatning", CompetitionCB, "", "help:4342");
      gdi.addButton("Paste", "Klistra in", CompetitionCB, "", "help:6666");
      gdi.addButton("ImportFile", "Importera fil...", CompetitionCB, "", "help:3029");
      gdi.addButton("ImportDB", "Bygg databaser...", CompetitionCB, "", "help:8079");
      gdi.fillDown();
    }
    else if(bi.id=="ImportDB") {
      if(!gdi.ask("help:146122"))
        return 0;

      string file=gdi.browseForOpen("XML-filer*.xml;*.meos", "xml");

      if(file.empty())
        return 0;
      gdi.setWaitCursor(true);
      oe->getFreeImporter(fi);
      string info;

      oe->importXMLNames(file.c_str(), fi, info);
      fi.save();
      gdi.alert(info);
      gdi.setWaitCursor(false);
    }
    else if(bi.id=="Paste") {
      gdi.pasteText("EntryText");
    }
    else if(bi.id=="ImportFile") {
      string file=gdi.browseForOpen("Textfiler*.txt", "txt");
      ifstream fin(file.c_str());
      char bf[1024];
      bf[0]='\r';
      bf[1]='\n';
      entryText.clear();
      while (fin.good() && !fin.eof()) {
        fin.getline(bf+2, 1024-2);
        entryText+=bf;
      }
      entryText+="\r\n";
      fin.close();

      gdi.setText("EntryText", entryText);
    }
    else if(bi.id=="PreviewImport") {
      oe->getFreeImporter(fi);
      entryText=gdi.getText("EntryText");
      gdi.clearPage(false);
      gdi.addString("", 2, "Frhandsgranskning, import");
      gdi.dropLine(0.5);
      char *bf=new char[entryText.length()+1];
      strcpy_s(bf, entryText.length()+1, entryText.c_str());      
      fi.extractEntries(bf, entries);
      delete[] bf;
      fi.showEntries(gdi, entries);
      gdi.fillRight();
      gdi.dropLine(1);
      gdi.addButton("DoFreeImport", "Spara anmlningar", CompetitionCB);
      gdi.addButton("FreeImport", "ndra", CompetitionCB);
      
      gdi.addButton("Cancel", "Avbryt", CompetitionCB);
      gdi.scrollToBottom();
    }
    else if (bi.id=="DoFreeImport") {
      fi.addEntries(oe, entries);
      entryText.clear();
      loadPage(gdi);
    }
		else if (bi.id=="Results") {
      save(gdi);
			int FilterIndex=1;
			string save=browseForSave("Webben (html)*.htmlOE Semikolonseparerad (csv)*.csv", "html", FilterIndex);

			if(save.length()>0)	{
				gdi.setWaitCursor(true);

        if(FilterIndex==1)
          throw std::exception("Anvnd listor");
				else
          oe->exportOEcsv(save.c_str());				
			}
		}
		else if(bi.id=="Startlist") {
      save(gdi);
			int FilterIndex=1;
			string save=browseForSave("Webben (html)*.htmlOE Semikolonseparerad (csv)*.csv", "html", FilterIndex);

			if(save.length()>0)	{
				gdi.setWaitCursor(true);
				gdi.alert("Not implemented");
			}
		}
		else if(bi.id=="Splits") {
      save(gdi);
			int FilterIndex=1;
			string save=browseForSave("IOF Results (XML)*.xml", "xml", FilterIndex);

			if(save.length()>0)	{
				gdi.setWaitCursor(true);
				oe->exportIOFSplits(save.c_str());
			}
		}
		else if (bi.id=="ExportCSV") {
      save(gdi);
			int FilterIndex=1;
			string save=browseForSave("OE2003 (CSV)*.csv", "csv", FilterIndex);

			if(save.length()>0)	{
				gdi.setWaitCursor(true);
				oe->exportOECSV(save.c_str());
			}
		}
		else if (bi.id=="SaveAs") {
      save(gdi);
			exportFileAs(hWndMain, gdi);
		}
		else if (bi.id=="Import") {
      //Import complete competition
			importFile(hWndMain, gdi);
			loadPage(gdi);	
		}
		else if(bi.id=="Save")
      save(gdi);
		else if (bi.id=="CloseCmp") {
			gdi.setWaitCursor(true);
			oe->save();
			oe->newCompetition("");
      gdi.setWindowTitle("");
			if(showConnectionPage)
        loadConnectionPage(gdi);
      else
        loadPage(gdi);
			gdi.setWaitCursor(false);
		}
		else if (bi.id=="Delete" && 
			gdi.ask("Vill du verkligen radera tvlingen?")) {
			
      if (oe->isClient())
        oe->dropDatabase();
			else if (!oe->deleteCompetition()) 
				gdi.alert("Operation failed. It is not possible to delete competitions on server");
			
      oe->clearListedCmp();
			oe->newCompetition("");
      gdi.setWindowTitle("");
			loadPage(gdi);	
		}
		else if(bi.id=="NewCmp") {
			oe->newCompetition("Ny tvling");
			gdi.setWindowTitle("");
      loadPage(gdi);
			return 0;			
		}
		else if (bi.id=="OpenCmp") {
			ListBoxInfo lbi;
      int id=0;
      bool frontPage=true;
			if (!gdi.getSelectedItem("CmpSel", &lbi)) {
        frontPage=false;
        if( gdi.getSelectedItem("ServerCmp", &lbi) )
          id=lbi.data;
        else if( gdi.getSelectedItem("LocalCmp", &lbi) )
          id=lbi.data;
      }
      else id=lbi.data;

      if(id==0)
        throw std::exception("Ingen tvling vald");

			gdi.setWaitCursor(true);
			if(!oe->open(id))
				gdi.alert("Kunde inte ppna tvlingen.");
			else {
				if(gSI) gSI->SetZeroTime(oe->getZeroTimeNum());

        gdi.setWindowTitle(oe->getTitleName());
        oe->updateTabs();

        if(frontPage)
				  loadPage(gdi);
        else {
          oe->verifyConnection();
          oe->validateClients();
          loadConnectionPage(gdi);
        }
        return 0;
			}
		}
		else if(bi.id=="BrowseCourse") {
			string file=browseForOpen("OCAD, banexport*.csv", "csv");
			if(file.length()>0)
				gdi.setText("FileName", file);
		}
		else if (bi.id=="BrowseEntries") {
			string file=browseForOpen("Importerbara*.xml;*.csvIOF-xml*.xmlSemikolonseparerad OE-standard*.csv", "xml");
			if(file.length()>0)
				gdi.setText("FileName", file);
		}
		else if(bi.id=="Entries") {
      if(!save(gdi))
        return 0;

			gdi.clearPage(true);
			gdi.addString("", 2, "Importera lpare");
			gdi.addString("", 0, "help:8180");
			gdi.dropLine();

			gdi.fillRight();
			gdi.pushX();
			gdi.addInput("FileName", "", 32, 0, "Filnamn");
			gdi.dropLine();
			gdi.fillDown();
			gdi.addButton("BrowseEntries", "Blddra...", CompetitionCB);
			
			gdi.popX();
			gdi.dropLine();
			gdi.pushX();
			gdi.fillRight();
			gdi.addButton("DoImport", "Importera", CompetitionCB);
			gdi.fillDown();			
			gdi.addButton("Cancel", "Avbryt", CompetitionCB);
			gdi.popX();
		}
		else if(bi.id=="DoImport") {
			csvparser csv;
			int type=csv.iscsv(gdi.getText("FileName").c_str());
			if(type)
			{
				gdi.disableInput("DoImport");
				gdi.disableInput("Cancel");
				gdi.disableInput("BrowseEntries");
        const char *File=gdi.getText("FileName").c_str();

				if (type==1) {
					gdi.addString("", 0, "Frsker importera OE2003 csv-fil...");
					gdi.refresh();
					gdi.setWaitCursor(true);
					if (csv.ImportOE_CSV(*oe, File))	{
						char bf[128];
						sprintf_s(bf, 128, "Klart. %d personer importerade", csv.nimport);
						gdi.addString("", 0, bf);
					}
					else gdi.addString("", 0, "Frsket misslyckades.");
				}
        else if (type==2) {
					gdi.addString("", 0, "Frsker importera OS2003 csv-fil...");
					gdi.refresh();
					gdi.setWaitCursor(true);
					if (csv.ImportOS_CSV(*oe, File)) {
						char bf[128];
						sprintf_s(bf, 128, "Klart. %d lag importerade", csv.nimport);
						gdi.addString("", 0, bf);
					}
					else gdi.addString("", 0, "Frsket misslyckades.");
        }
        else if (type==3) {
					gdi.addString("", 0, "Frsker importera RAID patrull csv-fil...");
					gdi.setWaitCursor(true);
					if (csv.ImportRAID(*oe, File)) {
						char bf[128];
						sprintf_s(bf, 128, "Klart. %d patruller importerade", csv.nimport);
						gdi.addString("", 0, bf);
					}
					else gdi.addString("", 0, "Frsket misslyckades.");

        }

				gdi.refresh();
				gdi.addButton("Cancel", "OK", CompetitionCB);				
			}
			else {
				//xmlparser xml;
				//xml.Read(gdi.getText("FileName").c_str());				
				oe->importXMLData(gdi.getText("FileName").c_str());
			}
		}
		else if (bi.id=="Courses") {
      if(!save(gdi))
        return 0;
			gdi.clearPage(true);
			gdi.addString("", 2, "Importera banor/klasser");
			gdi.addString("", 0, "help:5958");
			gdi.dropLine();

			gdi.fillRight();
			gdi.pushX();
			gdi.addInput("FileName", "", 32, 0, "Filnamn");
			gdi.dropLine();
			gdi.fillDown();
			gdi.addButton("BrowseCourse", "Blddra...", CompetitionCB);
			
			gdi.popX();
			gdi.pushX();
			gdi.dropLine();
			gdi.fillRight();
			gdi.addButton("DoImportCourse", "Importera", CompetitionCB);
			gdi.fillDown();					
			gdi.addButton("Cancel", "Avbryt", CompetitionCB);
			gdi.popX();
		}
		else if (bi.id=="DoImportCourse") {
			gdi.disableInput("DoImportCourses");
			gdi.disableInput("Cancel");
			gdi.disableInput("BrowseCourses");
			gdi.addString("", 0, "Frsker importera OCAD csv-fil...");
			gdi.refresh();
			
			csvparser csv;

			if(csv.ImportOCAD_CSV(*oe, gdi.getText("FileName").c_str())) {
				gdi.addString("", 0, "Klart.");
			}
			else gdi.addString("", 0, "Frsket misslyckades.");

			gdi.refresh();
			gdi.addButton("Cancel", "OK", CompetitionCB);			
		}
    else if (bi.id=="About") {
      loadAboutPage(gdi);
    }
	}
  else if(type==GUI_LISTBOX) {
    ListBoxInfo lbi=*(ListBoxInfo *)data;

    if(lbi.id=="LocalCmp") 
      gdi.selectItemByData("ServerCmp", -1);
    else if(lbi.id=="ServerCmp") 
      gdi.selectItemByData("LocalCmp", -1);
    else if (lbi.id=="TextSize") {
      int textSize = lbi.data;
      oe->setProperty("TextSize", textSize);
      gdi.setFont(textSize, "Arial");
      loadPage(gdi);
    }
  }
  else if (type==GUI_EVENT) {
    if( ((EventInfo*)data)->id=="Connections" ) {
      string s=gdi.getText("ClientName");
      loadConnectionPage(gdi);
      gdi.setText("ClientName", s);

    }
  }
  else if(type==GUI_CLEAR) {
    if (gdi.isInputChanged("")) {
      string name=gdi.getText("Name");

      if(!name.empty() && !oe->empty())
        save(gdi, false);
    }
    return 1;
	}
  return 0;
}

void TabCompetition::copyrightLine(gdioutput &gdi) const
{   
  gdi.pushX();
  gdi.fillRight();
 
  gdi.addButton("About", "Om MeOS...", CompetitionCB);

  gdi.dropLine(0.4);
  gdi.fillDown(); 
  gdi.addString("", 0, MakeDash("#Copyright  2007-2009 Melin Software HB"));
  char bf[256];
  gdi.dropLine(1);
  gdi.popX();

  sprintf_s(bf, "#Version 1.0.%d, %s", getMeosBuild(), getMeosDate().c_str());
  gdi.addString("", 0, bf).setColor(100,0,0);
  gdi.dropLine(0.2);
}

void TabCompetition::loadAboutPage(gdioutput &gdi) const
{
  gdi.clearPage(false);
  gdi.addString("", 2, MakeDash("Om MeOS - ett Mycket Enkelt OrienteringsSystem")).setColor(0,0,80);
  gdi.dropLine(2);
  gdi.addString("", 1, MakeDash("Copyright  2007-2009 Melin Software HB"));
  gdi.dropLine();
  gdi.addString("", 10, "The database connection used is MySQL++\nCopyright "
                        "(c) 1998 by Kevin Atkinson, (c) 1999, 2000 and 2001 by MySQL AB,"
                        "\nand (c) 2004-2007 by Educational Technology Resources, Inc.\n"
                        "The database used is MySQL, Copyright (c) 2008 Sun Microsystems, Inc.");
  
  gdi.dropLine();
  gdi.addString("", 0, "Det hr programmet levereras utan ngon som helst garanti. Programmet r ");
  gdi.addString("", 0, "fritt att anvnda och du r vlkommen att distribuera det under vissa villkor,");
  gdi.addString("", 0, "se license.txt som levereras med programmet.");
	
  gdi.dropLine();
  gdi.addButton("Cancel", "Stng", CompetitionCB);
  gdi.refresh();
}

bool TabCompetition::loadPage(gdioutput &gdi)
{
  showConnectionPage=false;
  oe->checkDB();
	gdi.clearPage(true);
	gdi.fillDown();

	if (oe->empty()) {
		gdi.addString("", 2, "Vlkommen till MeOS");
    gdi.addString("", 1, MakeDash("#- ")+ lang.tl("ett Mycket Enkelt OrienteringsSystem")).setColor(0,0,80);
		gdi.dropLine();
		
    gdi.addSelection("CmpSel", 300, 400, CompetitionCB, "Vlj tvling");
	
		char bf[260];
		GetUserFile(bf, "");
		oe->enumerateCompetitions(bf, "*.meos");		
		oe->fillCompetitions(gdi, "CmpSel",0);
		gdi.selectFirstItem("CmpSel");

		gdi.addButton("OpenCmp", "ppna", CompetitionCB);
		gdi.dropLine();

		gdi.pushX();
		gdi.fillRight();
		gdi.addButton("NewCmp", "Ny tvling", CompetitionCB);
		

		gdi.addButton("Import", "Importera tvling...", CompetitionCB);
	
		gdi.addButton("ConnectMySQL", "Databasanslutning...", CompetitionCB);

    gdi.popX();
    gdi.dropLine(3);
    gdi.addButton("Setup", "Instllningar...", CompetitionCB);
    
    
    gdi.dropLine(0.3);
    textSizeControl(gdi); 
    
    gdi.popX();
    gdi.fillDown();	
    gdi.dropLine(5);
    copyrightLine(gdi);
	}
	else {
		gdi.addString("", 3, "MeOS");
		gdi.dropLine();

    oe->syncronize();

		gdi.addInput("Name", oe->getName(), 24, 0, "Tvlingsnamn:");

		gdi.pushX();
		gdi.fillRight();
		gdi.addInput("Date", oe->getDate(), 8, 0, "Datum:");	
		gdi.fillDown();		
		gdi.addInput("ZeroTime", oe->getZeroTime(), 8, 0, "Nolltid:");

    if (oe->isClient()) 
      gdi.disableInput("ZeroTime");

		gdi.popX();
		gdi.addString("", 0, "help:5150");
  
    gdi.dropLine();

    gdi.fillDown();
    gdi.addCheckbox("UseEconomy", "Hantera klubbar och ekonomi", CompetitionCB, oe->useEconomy());
    
    gdi.dropLine(-1.1);
    gdi.addCheckbox("UseSpeaker", "Anvnd speakerstd", CompetitionCB, oe->GetEventDCI().getInt("UseSpeaker")!=0);

    //gdi.dropLine();
    textSizeControl(gdi);
  
    gdi.dropLine(3);
    gdi.popX();
    gdi.fillRight();
		gdi.addButton("Save", "Spara", CompetitionCB, "", 
          "MeOS sparar alla ndringar automatiskt nr det behvs.");
    
		gdi.addButton("SaveAs", "Exportera / Skerhetskopiera...", CompetitionCB);
		gdi.dropLine(3);
    gdi.popX();

		gdi.fillRight(); 
		gdi.addButton("CloseCmp", "Stng", CompetitionCB);
		gdi.addButton("Delete", "Radera", CompetitionCB);
		gdi.fillDown();
		gdi.addButton("ConnectMySQL", "Databasanslutning...", CompetitionCB);

    gdi.popX();

    //gdi.addButton("Reset", "terstll instllningar...", CompetitionCB);
    gdi.fillDown();
    gdi.popX();

		gdi.newColumn();

		gdi.addButton("Entries", "Importera anmlningar...", CompetitionCB);
		gdi.addButton("FreeImport", "Fri anmlningsimport...", CompetitionCB);
    gdi.addButton("Courses", "Importera banor...", CompetitionCB);
		gdi.dropLine();
		//gdi.addButton("Startlist", "Exportera startlista...", CompetitionCB);
		//gdi.addButton("Results", "Exportera resultat...", CompetitionCB);
		gdi.addButton("Splits", "Exportera strcktider...", CompetitionCB, "Exportera IOF XML");
		gdi.addButton("ExportCSV", "Exportera datafil...", CompetitionCB, "Exportera OE CSV");
		
		gdi.dropLine();
		gdi.addButton("DBaseIn", "Importera lpardatabas", CompetitionCB);
		gdi.addButton("ExportSetup", "Exportera lpardatabas", CompetitionCB);
		//.disableInput("DBaseIn");
		//gdi.disableInput("DBaseOut");

    gdi.dropLine();
    gdi.addButton("Settings", "Tvlingsinstllningar", CompetitionCB);
		gdi.addButton("Report", "Tvlingsrapport", CompetitionCB);
		
    gdi.popX();
    gdi.dropLine(4);
    copyrightLine(gdi);

    gdi.setOnClearCb(CompetitionCB);
	}
	gdi.refresh();
  return true;
}

void TabCompetition::textSizeControl(gdioutput &gdi) const 
{
  int s = oe->getPropertyInt("TextSize", 0);
  const char *id="TextSize";
  gdi.fillRight();
  gdi.addString("", 0, "Textstorlek:");
  gdi.dropLine(-0.2);
  gdi.addSelection(id, 90, 200, CompetitionCB);
  gdi.addItem(id, lang.tl("Normal"), 0);
  gdi.addItem(id, lang.tl("Stor"), 1);
  gdi.addItem(id, lang.tl("Strre"), 2);
  gdi.addItem(id, lang.tl("Strst"), 3);
  gdi.selectItemByData(id, s);
}
