/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

#include "stdafx.h"

#include "resource.h"

#include <commctrl.h>
#include <commdlg.h> 

#include "oEvent.h"
#include "xmlparser.h"
#include "gdioutput.h"
#include "csvparser.h"
#include "SportIdent.h"
#include "meos_util.h"

#include "TabSpeaker.h"
#include <cassert>
//static TabSpeaker *tabSpeaker=0;

//Base position for speaker buttons
#define SPEAKER_BASE_X 20

TabSpeaker::TabSpeaker(oEvent *poe):TabBase(poe)
{
  assert(!FixedTabs::speakerTab);
  FixedTabs::speakerTab=this;

  classId=0;
}

TabSpeaker::~TabSpeaker(void)
{
  FixedTabs::speakerTab=0;

}

void clearSpeakerData()
{
	if (!FixedTabs::speakerTab) 
		return;

  FixedTabs::speakerTab->clear();
}

int tabSpeakerCB(gdioutput *gdi, int type, void *data)
{
	if (!FixedTabs::speakerTab) 
		throw std::exception("tabSpeaker undefined.");

	switch(type){
		case GUI_BUTTON: {
			//Make a copy
			ButtonInfo bu=*static_cast<ButtonInfo *>(data);
			return FixedTabs::speakerTab->processButton(*gdi, bu);
  	}
		case GUI_LISTBOX:{	
			ListBoxInfo lbi=*static_cast<ListBoxInfo *>(data);
			return FixedTabs::speakerTab->processListBox(*gdi, lbi);
		}
	}
	return 0;
}

int TabSpeaker::processButton(gdioutput &gdi, const ButtonInfo &bu)
{
	if (bu.id=="Settings") {
    gdi.restore("settings");  
    gdi.fillDown();
    gdi.addString("", 0, "help:4948");
    gdi.pushX();
    gdi.fillRight();
    gdi.addListBox("Classes", 150, 300,0,"Klasser","", true);
    
    oe->fillClasses(gdi, "Classes", oEvent::extraNone, oEvent::filterNone);
    gdi.setSelection("Classes", classesToWatch);
   
    gdi.addListBox("Controls", 200, 300,0, "Kontroller","", true);
    gdi.pushX();
    gdi.fillDown();
    oe->fillControls(gdi, "Controls");
    gdi.setSelection("Controls", controlsToWatch);

    gdi.addButton("OK", "OK", tabSpeakerCB);
  }
  else if (bu.id=="Manual") {
    gdi.restore("settings");
    gdi.fillDown();
    gdi.addString("", 1, "Inmatning av mellantider");
    gdi.dropLine(0.5);
    manualTimePage(gdi);
  }
  else if (bu.id=="StoreTime") {
    storeManualTime(gdi);
  }
  else if (bu.id=="Cancel") {
    loadPage(gdi);
  }
  else if (bu.id=="OK") {
    gdi.getSelection("Classes", classesToWatch);  
    gdi.getSelection("Controls", controlsToWatch);  

    for (set<int>::iterator it=controlsToWatch.begin();it!=controlsToWatch.end();++it) {
      pControl pc=oe->getControl(*it, false);

      if (pc) 
        controlsToWatchSI.insert(pc->Numbers, pc->Numbers+pc->nNumbers);
    }

    loadPage(gdi);
  }
  else if (bu.id.substr(0, 3)=="cid" ) {
    classId=atoi(bu.id.substr(3, string::npos).c_str());
    generateControlList(gdi, classId);
		gdi.setRestorePoint("speaker");
  	gdi.setRestorePoint("SpeakerList");

    if (selectedControl.count(classId)==1)
      oe->speakerList(gdi, classId, selectedControl[classId].getLeg(), 
                                    selectedControl[classId].getControl()); 
  }
  else if (bu.id.substr(0, 4)=="ctrl") {
    int ctrl=atoi(bu.id.substr(4, string::npos).c_str());    
    selectedControl[classId].setControl(ctrl);
    gdi.restore("speaker");
    oe->speakerList(gdi, classId, selectedControl[classId].getLeg(), ctrl);    
  }
  
  return 0;
}

void TabSpeaker::generateControlList(gdioutput &gdi, int classId)
{
  pClass pc=oe->getClass(classId);  

  if(!pc)
    return;

  int leg = selectedControl[pc->getId()].getLeg();

  list<pControl> controls;
  
  gdi.restore("classes", true);
  pCourse course=0;      

  if(pc->hasMultiCourse())
    course=pc->getCourse(leg);
  else
    course=pc->getCourse();

  if(course)						
	  course->getControls(controls);						
  
  list<pControl>::iterator it;
	gdi.fillDown();

  int h,w;
  gdi.getTargetDimension(w, h);

  int bw=gdi.scaleLength(100);
  int nbtn=max((w-80)/bw, 1);
  bw=(w-80)/nbtn;
  int basex=SPEAKER_BASE_X;
  int basey=gdi.getCY()+4;

  int cx=basex;
  int cy=basey;
  int cb=1;

  if (pc->getNumStages()>1) {
    gdi.addSelection(cx, cy+2, "Leg", bw-5, 100, tabSpeakerCB);
    char bf[64];

    for (size_t k=0;k<pc->getNumStages();k++) {
      sprintf_s(bf, lang.tl("Strcka %d").c_str(), k+1);
      gdi.addItem("Leg", bf, k);
    }
    gdi.selectItemByData("Leg", leg);
    cb+=1;
    cx+=1*bw;
  }

	for (it=controls.begin(); it!=controls.end(); ++it) {		
    if (controlsToWatch.count( (*it)->getId() ) ) {

		  char bf[16];
      sprintf_s(bf, "ctrl%d", (*it)->getId());
      gdi.addButton(cx, cy, bw, bf, (*it)->getName(), tabSpeakerCB);
      cx+=bw;
      
      cb++;
      if (cb>nbtn) {
        cb=1;
        cy+=gdi.getButtonHeight()+4;
        cx=basex;
      }
    }
  }
	gdi.fillDown();
  char bf[16];
  sprintf_s(bf, "ctrl%d", oPunch::PunchFinish);
  gdi.addButton(cx, cy, bw, bf, "Ml", tabSpeakerCB);

  gdi.popX(); 
}

int TabSpeaker::processListBox(gdioutput &gdi, const ListBoxInfo &bu)
{
  if(bu.id=="Leg") {
    if (classId>0) {
      selectedControl[classId].setLeg(bu.data);
      generateControlList(gdi, classId);
      oe->speakerList(gdi, classId, selectedControl[classId].getLeg(),
                      selectedControl[classId].getControl());    
    }
  }

  return 0;
}

bool TabSpeaker::loadPage(gdioutput &gdi)
{
  oe->checkDB();

	gdi.clearPage(true);
  gdi.pushX();
  gdi.setRestorePoint("settings");

  gdi.pushX();
  gdi.fillDown();

  int h,w;
  gdi.getTargetDimension(w, h);

  int bw=gdi.scaleLength(100);
  int nbtn=max((w-80)/bw, 1);
  bw=(w-80)/nbtn;
  int basex=SPEAKER_BASE_X;
  int basey=gdi.getCY();

  int cx=basex;
  int cy=basey;
  int cb=1;
  for (set<int>::iterator it=classesToWatch.begin();it!=classesToWatch.end();++it) {
    char classid[32];
    sprintf_s(classid, "cid%d", *it);
    pClass pc=oe->getClass(*it);
    
    if (pc) {
      gdi.addButton(cx, cy, bw, classid, pc->getName(), tabSpeakerCB);
      cx+=bw;
      cb++;

      if (cb>nbtn) {
        cb=1;
        cx=basex;
        cy+=gdi.getButtonHeight()+4;
      }
    }
  }

  if (classesToWatch.empty()) {
    gdi.addString("", boldLarge, "Speakerstd"); 
    //gdi.dropLine();
    //gdi.addString("", 10, "Hjlp fr speakrar och speakers...\n\nJust ni r speakerklienten i dligt skick. terkommer i senare release.");
    gdi.dropLine();
    cy=gdi.getCY();
    cx=gdi.getCX();
  }
  gdi.addButton(cx, cy, bw, "Settings", "Instllningar...", tabSpeakerCB);
  gdi.addButton(cx+bw, cy, bw, "Manual", "Tidsinmatning", tabSpeakerCB);
 
  gdi.setRestorePoint("classes");

	return true;
}

void TabSpeaker::clear()
{
  controlsToWatch.clear();
  classesToWatch.clear();
  controlsToWatchSI.clear();
  selectedControl.clear();
  classId=0;
  lastControl.clear();
}

void TabSpeaker::manualTimePage(gdioutput &gdi) const
{
  gdi.setRestorePoint("manual");

  gdi.fillRight();
  gdi.pushX();
  gdi.addInput("Control", lastControl, 5, 0, "Kontroll");
  gdi.addInput("Runner", "", 6, 0, "Lpare");
  gdi.addInput("Time", "", 8, 0, "Tid");
  gdi.dropLine();
  gdi.addButton("StoreTime", "Spara", tabSpeakerCB).setDefault();
  gdi.addButton("Cancel", "Avbryt", tabSpeakerCB).setDefault();
  gdi.fillDown();
  gdi.popX();
  gdi.dropLine(3);
  gdi.addString("", 10, "help:14692");


  gdi.setInputFocus("Runner");
  gdi.refresh();
}

void TabSpeaker::storeManualTime(gdioutput &gdi) 
{
  char bf[256];

  int punch=gdi.getTextNo("Control");
  
  if (punch<=0) 
    throw std::exception("Kontrollnummer mste anges.");

  lastControl=gdi.getText("Control");
  int r_no=gdi.getTextNo("Runner");
  string time=gdi.getText("Time");
  pRunner r=oe->getRunnerByStartNo(r_no);

  if(!r)
    r=oe->getRunnerByCard(r_no);

  string Name;
  int sino=r_no;
  if(r) {
    Name=r->getName();
    sino=r->getCardNo();
  }
  else
    Name="Oknd";

  if (time.empty())
    time=getLocalTimeOnly();
  
  int itime=oe->getRelativeTime(time);

  if (itime>0 && sino>0) {
    pFreePunch fp=oe->addFreePunch(itime, punch, sino);
    fp->syncronize();

    sprintf_s(bf, "Lpare: %s, kontroll: %d, kl %s", 
      Name.c_str(), punch, oe->getAbsTime(itime).c_str());
                  
    gdi.restore("manual", false);

    gdi.addString("", 0, bf);
  }
  else
    throw std::exception("Ogiltig tid.");

  manualTimePage(gdi);
}
