/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

// oClub.cpp: implementation of the oClub class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "oClub.h"
#include "meos_util.h"

#include "oEvent.h"
#include "gdioutput.h"
#include "RunnerDB.h"
#include <cassert>
#include "Table.h"
#include "localizer.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

oClub::oClub(oEvent *poe): oBase(poe)
{
	getDI().initData();
	Id=oe->getFreeClubId();
}

oClub::oClub(oEvent *poe, int id): oBase(poe)
{
	getDI().initData();
	Id=id;
  oe->qFreeClubId = max(id, oe->qFreeClubId);
}


oClub::~oClub()
{

}

bool oClub::Write(xmlparser &xml)
{
	if(Removed) return true;

	xml.StartTag("Club");
	xml.Write("Id", Id);
	xml.Write("Updated", Modified.GetStamp());
	xml.Write("Name", Name);
	getDI().write(xml);

	xml.EndTag();

	return true;
}

void oClub::Set(const xmlobject *xo)
{
	const xmlList *xl=xo->GetObjects();

	if(xl){
		xmlList::const_iterator it;

		for(it=xl->begin(); it != xl->end(); ++it){
			if(it->Is("Id")){
				Id=atoi(it->Get().c_str());			
			}
			else if(it->Is("Name")){
				Name=it->Get();
			}
			else if(it->Is("oData")){
				getDI().set(&*it);
			}
			else if(it->Is("Updated")){
				Modified.SetStamp(it->Get());
			}
		}
	}
}

void oClub::setName(string n)
{
	if(n!=Name){
		Name=n;
		Modified.Update();
	}
}
oDataConstInterface oClub::getDCI(void) const
{
	return oe->oClubData->getConstInterface(oData, sizeof(oData), this);
}

oDataInterface oClub::getDI(void)
{
	return oe->oClubData->getInterface(oData, sizeof(oData), this);
}


pClub oEvent::getClub(int Id) const
{
  map<int, pClub>::const_iterator mit=clubIdIndex.find(Id);

  if (mit!=clubIdIndex.end()) 
    return mit->second;

  return 0;
}

pClub oEvent::getClub(const string &pname) const
{
	oClubList::const_iterator it;	

	for (it=Clubs.begin(); it != Clubs.end(); ++it)
		if(it->Name==pname)	
      return pClub(&*it);
	
  return 0;
}

pClub oEvent::getClubCreate(int Id, const string &CreateName) 
{
  map<int, pClub>::iterator mit=clubIdIndex.find(Id);
  if (mit!=clubIdIndex.end()) {
    if (!trim(CreateName).empty() && _stricmp(mit->second->getName().c_str(), trim(CreateName).c_str())!=0)
      Id = 0; //Bad, used Id.
    if (trim(CreateName).empty() || Id>0)
      return mit->second;
  }
  if (CreateName.empty()) {    
		//Not found. Auto add...	
		return getClubCreate(Id, "Klubbls");
	}
	else	{
    oClubList::iterator it;
    string tName = trim(CreateName);

		//Maybe club exist under different ID
		for (it=Clubs.begin(); it != Clubs.end(); ++it)
      if(_stricmp(it->Name.c_str(), tName.c_str())==0)
        return &*it;
	
		//Else, create club.
		return addClub(tName, Id);
	}
}

pClub oEvent::addClub(const string &pname, int createId)
{	
  if (createId>0) {
    pClub pc = getClub(createId);
    if (pc)
      return pc;
  }

  pClub dbClub = oe->runnerDB->getClub(pname);

  if (dbClub) {
    if (createId<=0)
      if (getClub(dbClub->Id))
        createId = getFreeClubId(); //We found a db club, but Id is taken.
      else
        createId = dbClub->Id;

    oClub c(this, createId);
    c = *dbClub;
    c.Id = createId;
    Clubs.push_back(c);
  }
  else {
    if (createId==0)
      createId = getFreeClubId();
  
    oClub c(this, createId);
    c.setName(pname);
	  Clubs.push_back(c);
  }
  Clubs.back().syncronize();
  clubIdIndex[Clubs.back().Id]=&Clubs.back();
  return &Clubs.back();
}

pClub oEvent::addClub(const oClub &oc)
{	
  if(clubIdIndex.count(oc.Id)!=0)
    return clubIdIndex[oc.Id];

	Clubs.push_back(oc);	
  if (!oc.existInDB())
    Clubs.back().syncronize();

  clubIdIndex[oc.Id]=&Clubs.back();
	return &Clubs.back();
}

bool oEvent::fillClubs(gdioutput &gdi, const string &name)
{	
	gdi.clearList(name);
	syncronizeList(oLClubId);
  Clubs.sort();

	oClubList::iterator it;	

	for (it=Clubs.begin(); it != Clubs.end(); ++it){
		if(!it->Removed)
			gdi.addItem(name, it->Name, it->Id);
	}

	return true;
}

Table *oEvent::getClubsTB()//Table mode
{	
	Table *table=new Table(this, 20, "Klubbar", "clubs");
	table->addColumn("Namn", 200, false);
  oe->oClubData->buildTableCol(table);
  table->update();
  
  return table;
}

void oEvent::generateClubTableData(Table *table)
{ 
  syncronizeList(oLClubId);
	oClubList::iterator it;	

  for (it=Clubs.begin(); it != Clubs.end(); ++it){		
    if(!it->isRemoved()){
      table->addRow(it->getId());					
      table->set(0, *it, TID_CLUB, it->getName(), true);
				
      oe->oClubData->fillTableCol(it->oData, *it, table);
		}
	}
}

bool oClub::inputData(int id, const string &input, 
                        int inputId, string &output, bool noUpdate)
{
  if(id>1000)
    return oe->oClubData->inputData(oData, id, input, output, noUpdate);

  switch(id) {
    case TID_CLUB:
      setName(input);
      syncronize();
      output=getName();
      return true;
    break;
  }

  return false;
}

void oClub::fillInput(int id, gdioutput &gdi, const string &controlId)
{ 
}

void oEvent::mergeClub(int clubIdPri, int clubIdSec)
{
  if (clubIdPri==clubIdSec)
    return;

  pClub pc = getClub(clubIdPri);
  if (!pc)
    return;

  // Update teams
  for (oTeamList::iterator it = Teams.begin(); it!=Teams.end(); ++it) {
    if (it->getClubId() == clubIdSec) {
      it->Club = pc;
      it->updateChanged();      
      it->syncronize(); 
    }
  }

  // Update runners
  for (oRunnerList::iterator it = Runners.begin(); it!=Runners.end(); ++it) {
    if (it->getClubId() == clubIdSec) {
      it->Club = pc;
      it->updateChanged();
      it->syncronize(); 
    }
  }
  oe->removeClub(clubIdSec);
}


void oEvent::viewClubMembers(gdioutput &gdi, int clubId)
{
  sortRunners(ClassStartTime);
  sortTeams(ClassStartTime, 0);
  
  gdi.fillDown();
  gdi.dropLine();
  int nr = 0;
  int nt = 0;
  // Update teams
  for (oTeamList::iterator it = Teams.begin(); it!=Teams.end(); ++it) {
    if (it->getClubId() == clubId) {
      if (nt==0)
        gdi.addString("", 1, "Lag");
      gdi.addStringUT(0, it->getName() + ", " + it->getClass() );
      nt++;
    }
  }

  gdi.dropLine();
  // Update runners
  for (oRunnerList::iterator it = Runners.begin(); it!=Runners.end(); ++it) {
    if (it->getClubId() == clubId) {
      if (nr==0)
        gdi.addString("", 1, "Lpare:");
      gdi.addStringUT(0, it->getName() + ", " + it->getClass() );
      nr++; 
    }
  }
}

void oClub::generateInvoice(gdioutput &gdi, int number, int &toPay, int &hasPaid) const
{
  toPay = 0;
  vector<pRunner> runners;
  oe->getClubRunners(getId(), runners);

  if (runners.empty())
    return;

  int xs = 20;
  int ys = gdi.getCY();
  int lh = gdi.getLineHeight();

  gdi.addString("", ys, xs+350, boldHuge, "FAKTURA");
  if (number>0)
    gdi.addStringUT(ys+lh*3, xs+350, fontMedium, lang.tl("Faktura nr")+ ": " + itos(number));
  int yp = ys+lh;
  string organizer = oe->GetEventDI().getString("Organizer");
  string ostreet = oe->GetEventDI().getString("Street");
  string oaddress = oe->GetEventDI().getString("Address");
  
  if (!organizer.empty())
    gdi.addStringUT(yp, xs, fontMedium, organizer), yp+=lh;
  if (!ostreet.empty())
    gdi.addStringUT(yp, xs, fontMedium, ostreet), yp+=lh;
  if (!oaddress.empty())
    gdi.addStringUT(yp, xs, fontMedium, oaddress), yp+=lh;

  yp+=lh;

  gdi.addStringUT(yp, xs, fontLarge, oe->getName());
  gdi.addStringUT(yp+lh*2, xs, fontMedium, oe->getDate());

  string co =  getDCI().getString("CareOf");
  string address =  getDCI().getString("Street");
  string city =  getDCI().getString("ZIP") + " " + getDCI().getString("City");
  
  int ayp = ys + 122;  
  gdi.addStringUT(ayp, xs+350, fontMedium, getName());
  ayp+=lh;

  if (!co.empty())
    gdi.addStringUT(ayp, xs+350, fontMedium, co), ayp+=lh;

  if (!address.empty())
    gdi.addStringUT(ayp, xs+350, fontMedium, address), ayp+=lh;

  if (!city.empty())
    gdi.addStringUT(ayp, xs+350, fontMedium, city), ayp+=lh;

  yp = ayp+30;

  int total_fee_amount = 0;
  int total_rent_amount = 0;
  int total_paid_amount = 0;
  char bf[64];

  gdi.addString("", yp, xs, fontSmall, "Deltagare");
  gdi.addString("", yp, xs+300, fontSmall, "Klass");
  
  gdi.addString("", yp, xs+430, fontSmall, "Resultat");
  
  gdi.addString("", yp, xs+580, fontSmall|textRight, "Avgift");
  gdi.addString("", yp, xs+630, fontSmall|textRight, "Brickhyra");
  gdi.addString("", yp, xs+680, fontSmall|textRight, "Betalat");
  yp += lh;

  for (size_t k=0;k<runners.size(); k++) {
    gdi.addStringUT(yp, xs, fontMedium, runners[k]->getName());
    gdi.addStringUT(yp, xs+300, fontMedium, runners[k]->getClass());
    string ts;
    
    if (runners[k]->getStatus()==StatusUnknown)
      ts = "-";      
    else if (runners[k]->getStatus()==StatusOK) {
      ts =  runners[k]->getPrintPlaceS()+ " (" + runners[k]->getRunningTimeS() + ")";
    } 
    else
      ts =  runners[k]->getStatusS();

    gdi.addStringUT(yp, xs+430, fontMedium, ts);

    int fee = runners[k]->getDCI().getInt("Fee");
    int card = runners[k]->getDCI().getInt("CardFee");
    int paid = runners[k]->getDCI().getInt("Paid");
    gdi.addStringUT(yp, xs+580, fontMedium|textRight, itos(fee));
    gdi.addStringUT(yp, xs+630, fontMedium|textRight, itos(card));
    gdi.addStringUT(yp, xs+680, fontMedium|textRight, itos(paid));
    total_fee_amount += fee;
    total_rent_amount += card;
    total_paid_amount += paid;
    yp += lh;
  }
  yp += lh;
  gdi.addStringUT(yp, xs+580, boldText|textRight, itos(total_fee_amount));
  gdi.addStringUT(yp, xs+630, boldText|textRight, itos(total_rent_amount));
  gdi.addStringUT(yp, xs+680, boldText|textRight, itos(total_paid_amount));

  yp+=lh*2;
  toPay = total_fee_amount+total_rent_amount-total_paid_amount;
  hasPaid = total_paid_amount;
  sprintf_s(bf, lang.tl("Att betala: %d").c_str(), toPay);
  gdi.addStringUT(yp, xs+600, boldText, bf);
  gdi.UpdatePos(700,0,0,0);

  yp+=lh*2;

  string account = oe->GetEventDI().getString("Account");
  string pdate = oe->GetEventDI().getDate("PaymentDue");
  

  gdi.addStringUT(yp, xs,boldText, lang.tl("Vnligen betala senast") + 
            " " + pdate + " " + lang.tl("till") + " " + account + ".");
  gdi.dropLine(2);
  gdi.addStringUT(gdi.getCY()-1, 0, pageNewPage, "");
}

void oEvent::getClubRunners(int clubId, vector<pRunner> &runners) const
{
  oRunnerList::const_iterator rit;
  runners.clear();

	for (rit=Runners.begin(); rit != Runners.end(); ++rit) {
    if (!rit->skip() && rit->getClubId() == clubId)
      runners.push_back(pRunner(&*rit));
	}  
}

void oEvent::printInvoices(gdioutput &gdi, bool onlySummary) 
{
  Clubs.sort();
  oClubList::const_iterator it;
  oe->calculateResults();
  oe->sortRunners(ClassStartTime);
  int pay, paid;
  vector<int> fees;
  vector<int> vpaid;
  fees.reserve(Clubs.size());
  int k=0;
	for (it=Clubs.begin(); it != Clubs.end(); ++it) {
    if (!it->isRemoved()) {
      it->generateInvoice(gdi, 1000+(k++), pay, paid);
      fees.push_back(pay);
      vpaid.push_back(paid);
    }
	}

  if (onlySummary)
    gdi.clearPage(true);
  k=0;
  gdi.dropLine(1);
  gdi.addString("", boldLarge, "Sammanstllning, ekonomi");
  int yp = gdi.getCY() + 10;

  gdi.addString("", yp, 50, boldText, "Faktura nr");
  gdi.addString("", yp, 240, boldText|textRight, "KlubbId"); 
  
  gdi.addString("", yp, 250, boldText, "Klubb"); 
  gdi.addString("", yp, 550, boldText|textRight, "Faktura"); 
  gdi.addString("", yp, 620, boldText|textRight, "Kontant"); 

  yp+=gdi.getLineHeight()+3;

  int sum = 0, psum = 0;
  for (it=Clubs.begin(); it != Clubs.end(); ++it) {
    if (!it->isRemoved()) {
      
      gdi.addStringUT(yp, 50, fontMedium, itos(1000+k));
      gdi.addStringUT(yp, 240, textRight|fontMedium, itos(it->getId())); 
      gdi.addStringUT(yp, 250, fontMedium, it->getName()); 
      gdi.addStringUT(yp, 550, fontMedium|textRight, itos(fees[k])); 
      gdi.addStringUT(yp, 620, fontMedium|textRight, itos(vpaid[k])); 
      
      sum+=fees[k];
      psum+=vpaid[k];
      k++;
      yp+=gdi.getLineHeight();
    }
	}

  gdi.addStringUT(yp, 550, boldText|textRight, lang.tl("Totalt faktureras")+": "+itos(sum));
  gdi.addStringUT(yp+gdi.getLineHeight(), 550, boldText|textRight, lang.tl("Totalt kontant")+": "+itos(psum));
}

void oClub::updateFromDB()
{
  pClub pc = oe->runnerDB->getClub(Id);

  if (pc && !pc->sameClub(*this))
    pc = 0;

  if (pc==0)
    pc = oe->runnerDB->getClub(Name);

  if (pc) {
    memcpy(oData, pc->oData, sizeof (oData));
    updateChanged();
  }
}

void oEvent::updateClubsFromDB()
{
  oClubList::iterator it;

  for (it=Clubs.begin();it!=Clubs.end();++it) {
    it->updateFromDB();
    it->syncronize();   
  }
}
