// oControl.h: interface for the oControl class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_OCONTROL_H__E86192B9_78D2_4EEF_AAE1_3BD4A8EB16F0__INCLUDED_)
#define AFX_OCONTROL_H__E86192B9_78D2_4EEF_AAE1_3BD4A8EB16F0__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "xmlparser.h"
#include "oBase.h"
#include <limits>

class oControl;

typedef oControl* pControl;
class oDataInterface;
class oDataConstInterface;

class oControl : public oBase 
{
public:
  enum ControlStatus {StatusOK=0, StatusBad=1, StatusMultiple=2};
  bool operator<(const oControl &b) const {return minNumber()<b.minNumber();}
	
protected:
	int nNumbers;
	int Numbers[32];
  bool checkedNumbers[32];

	ControlStatus Status;
	string Name;
	bool decodeNumbers(string s);
	BYTE oData[16];
public:
  string getInfo() const;

  inline int minNumber() const {
    int m = numeric_limits<int>::max();
    for (int k=0;k<nNumbers;k++)
      m = min(Numbers[k], m);
    return m;
  }

  inline int maxNumber() const {
    int m = 0;
    for (int k=0;k<nNumbers;k++)
      m = max(Numbers[k], m);
    return m;
  }

  //Add unchecked controls to the list
  void addUncheckedPunches(vector<int> &mp) const;
  //Start checking if all punches needed for this control exist
  void startCheckControl();
  //Get the number of a missing punch 
  int getMissingNumber() const;
  //Returns true if the check of this control is completed
  bool controlCompleted() const;

	oDataInterface getDI(void);
  oDataConstInterface getDCI(void) const;

	string codeNumbers(char sep=';');
	bool setNumbers(const string &numbers);

	ControlStatus getStatus() const {return (ControlStatus)Status;}

  bool hasName() const {return !Name.empty();}
	string getName() const;

	void setStatus(ControlStatus st);
	void setName(string name);

  //Returns true if control has number and checks it.
	bool hasNumber(int i);
  //Return true if it has number i and it is unchecked.
  //Checks the number
	bool hasNumberUnchecked(int i);
	
  string getString();
	string getLongString();

  //For a control that requires several punches,
  //return the number of required punches.
  int getNumMulti();

  int getTimeAdjust() const;
  string getTimeAdjustS() const;
  void setTimeAdjust(int v);
  void setTimeAdjust(const string &t);

	void set(const xmlobject *xo);
	void set(int pId, int pNumber, string pName);
	bool write(xmlparser &xml);
	oControl(oEvent *poe);
	oControl(oEvent *poe, int id);

  virtual ~oControl();

	friend class oRunner;
	friend class oCourse;
	friend class oEvent;	
	friend class MeosSQL;
  friend class TabAuto;
  friend class TabSpeaker;
};

#endif // !defined(AFX_OCONTROL_H__E86192B9_78D2_4EEF_AAE1_3BD4A8EB16F0__INCLUDED_)
