/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

// oEvent.cpp: implementation of the oEvent class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include <vector>
#include <io.h>
#include <algorithm>

#include "oEvent.h"
#include "oSpeaker.h"
#include "gdioutput.h"

#include "SportIdent.h"
#include "mmsystem.h"
#include "meos_util.h"
#include "localizer.h"
#include "meos.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//Order by preliminary times and priotity for speaker list
bool CompareSpkSPList(const oSpeakerObject &a, const oSpeakerObject &b)
{
	if(a.priority!=b.priority)
		return a.priority>b.priority;
	else if(a.status<=1 && b.status<=1){
    int at=a.preliminaryRunningTime;
		int bt=b.preliminaryRunningTime;

    if (at==bt) { //Compare leg times instead
      at=a.preliminaryRunningTimeLeg;
		  bt=b.preliminaryRunningTimeLeg;
    }

		if(at==bt)
			return a.name<b.name;
		else if(at>=0 && bt>=0)
			return at<bt;
		else if(at>=0 && bt<0) 
			return true;
		else if(at<0 && bt>=0)
			return false;
		else return at>bt;
	}
	else if(a.status!=b.status)
		return a.status<b.status;
	else return a.name<b.name;
}

//Order by known time for calculating results
bool CompareSOResult(const oSpeakerObject &a, const oSpeakerObject &b)
{
	if(a.status!=b.status){
		if(a.status==StatusOK)
			return true;
		else if(b.status==StatusOK)
			return false;
		else return a.status<b.status;
	}
	else if(a.status==StatusOK){
    int at=a.runningTime;
		int bt=b.runningTime;
    if(at!=bt)
		  return at<bt;
    else {
      at=a.runningTimeLeg;
		  bt=b.runningTimeLeg;
      if(at!=bt)
		    return at<bt;
      return a.name<b.name;
    }
	}
	else return a.name<b.name;
}


int SpeakerCB (gdioutput *gdi, int type, void *data)
{
	oEvent *oe=0;
	gdi->getData("oEvent", *LPDWORD(&oe));

	if(!oe)
		return false;

	DWORD ClassId=0, ControlId=0, leg=0;
	gdi->getData("ClassId", ClassId);
  gdi->getData("ControlId", ControlId);
  gdi->getData("LegNo", leg);

  if (ClassId>0 && ControlId>0) {
    pClass pc = oe->getClass(ClassId);
    if (pc && pc->wasSQLChanged())
		  oe->speakerList(*gdi, ClassId, leg, ControlId);
  }

	return true;
}

int MovePriorityCB(gdioutput *gdi, int type, void *data)
{
	if(type==GUI_LINK){

		oEvent *oe=0;
		gdi->getData("oEvent", *LPDWORD(&oe));
		if(!oe)	return false;

		TextInfo *ti=(TextInfo *)data;
		//gdi->alert(ti->id);
		if(ti->id.size()>1){
			//int rid=atoi(ti->id.substr(1).c_str());
      oRunner *r=static_cast<oRunner *>(ti->getExtra());

			if(!r) return false;
			
			DWORD ClassId=0, ControlId=0;
			if(gdi->getData("ClassId", ClassId) && gdi->getData("ControlId", ControlId)){			
			
				if(ti->id[0]=='D'){
					r->SetPriority(ControlId, -1);
				}
				else if(ti->id[0]=='U'){
					r->SetPriority(ControlId, 1);
				}
				else if(ti->id[0]=='M'){
					r->SetPriority(ControlId, 0);
				}

        int xo=gdi->GetOffsetX();
        int yo=gdi->GetOffsetY();
        gdi->SetOffsetX(0);
        gdi->SetOffsetY(0);
				gdi->restore("SpeakerList", false);        
				oe->speakerList(*gdi, ClassId, 0, ControlId);
        gdi->setOffset(xo, yo);
			}
		}
	}
	return true;
}

void oEvent::renderRowSpeakerList(gdioutput &gdi, oSpeakerObject &r, 
                                  oSpeakerObject *next_r, int x, int y, 
                                  int leaderTime, int type)
{
	const int dx_c[7]={0, 40, 300, 530, 590, 650, 660};
  int dx[7];
  for (int k=0;k<7;k++)
    dx[k]=gdi.scaleLength(dx_c[k]);

	int lh=gdi.getLineHeight();

	r.isRendered=true;
	gdi.addStringUT(y, x+dx[0], 0, r.placeS); 

	if(r.finishStatus<=1 || r.finishStatus==r.status)
		gdi.addStringUT(y, x+dx[1], 0, r.name, dx[2]-dx[1]-4);
	else
    gdi.addStringUT(y, x+dx[1], 0, r.name+ " ("+ oEvent::formatStatus(r.finishStatus) +")", dx[2]-dx[1]-4);

	gdi.addStringUT(y, x+dx[2], 0, r.club, dx[4]-dx[2]-4);
	
	if (r.status==StatusOK) {
		gdi.addStringUT(y, x+dx[4], textRight, formatTime(r.runningTime));
    
    if (r.runningTime != r.runningTimeLeg)
      gdi.addStringUT(y, x+dx[3], textRight, formatTime(r.runningTimeLeg));
    
		if(leaderTime!=100000){								
			gdi.addStringUT(y, x+dx[5], textRight, 
                    gdi.getTimerText(r.runningTime-leaderTime, 
                    timerCanBeNegative));
		}
	}
	else if (r.status==StatusUnknown) {		
		DWORD TimeOut=NOTIMEOUT;

    if (r.preliminaryRunningTimeLeg>0 && !r.missingStartTime) {
			
			if(next_r && next_r->status==StatusOK && 
          next_r->preliminaryRunningTime>r.preliminaryRunningTime)
				TimeOut=next_r->preliminaryRunningTime;
			
			RECT rc;
			rc.left=x+dx[1]-4;
			rc.right=x+dx[5]+60;
			rc.top=y-1;
			rc.bottom=y+lh+1;

			gdi.addRectangle(rc, -1, false);

			gdi.addTimer(y, x+dx[4], textRight, r.preliminaryRunningTime, 0, 0, TimeOut);

      if (r.preliminaryRunningTime != r.preliminaryRunningTimeLeg)
        gdi.addTimer(y, x+dx[3], textRight, r.preliminaryRunningTimeLeg, 0, 0, TimeOut);

			if(leaderTime!=100000)
				gdi.addTimer(y, x+dx[5], timerCanBeNegative|textRight, r.preliminaryRunningTime-leaderTime);
		}
		else{
			gdi.addStringUT(y, x+dx[4], textRight, "["+r.startTimeS+"]");
      if (!r.missingStartTime)
			  gdi.addTimer(y, x+dx[5], timerCanBeNegative|textRight, r.preliminaryRunningTimeLeg, 0, 0, 0);
		}
	}
	else{
    gdi.addStringUT(y, x+dx[4], textRight, oEvent::formatStatus(r.status));
	}
	char bf[16];
  int ownerId = r.owner ? r.owner->getId(): 0;

	if (type==1) {
    sprintf_s(bf, "D%d", ownerId);
		gdi.addString(bf, y, x+dx[6], 0, "[Flytta ner]", 0, MovePriorityCB).setExtra(r.owner);
	}
	else if (type==2) {
    sprintf_s(bf, "U%d", ownerId);
		gdi.addString(bf, y, x+dx[6], 0, "[Bevaka]", 0, MovePriorityCB).setExtra(r.owner);
	}
	else if (type==3) {
		if(r.status<=StatusOK){
			
			if(r.priority<0){ 
				sprintf_s(bf, "M%d", ownerId);
				gdi.addString(bf, y, x+dx[6], 0, "[terstll]", 0, MovePriorityCB).setExtra(r.owner);
			}
			else{
				sprintf_s(bf, "U%d", ownerId);
        gdi.addString(bf, y, x+dx[6], 0, "[Bevaka]", 0, MovePriorityCB).setExtra(r.owner);
			}
		}
	}

}

void oEvent::calculateResults(list<oSpeakerObject> &rl)
{
  rl.sort(CompareSOResult);	
	list<oSpeakerObject>::iterator it;

	int cPlace=0;
	int vPlace=0;
	int cTime=0;

	for (it=rl.begin(); it != rl.end(); ++it) {
    if (it->status==StatusOK) {
			cPlace++;

      if(it->runningTime>cTime)
				vPlace=cPlace;

      cTime=it->runningTime;

			it->place=vPlace;
		}
		else
			it->place=99000+it->status;
  
    char bf[16]="";
    if(it->place<90000)
      sprintf_s(bf, "%d", it->place);

    it->placeS=bf;
	}
}

void oEvent::speakerList(gdioutput &gdi, int ClassId, int leg, int ControlId)
{
	gdi.setData("ClassId", ClassId);
	gdi.setData("ControlId", ControlId);
  gdi.setData("LegNo", leg);
	gdi.setData("oEvent", DWORD(this));

	gdi.registerEvent("DataUpdate", SpeakerCB);
	gdi.setData("DataSync", 1);
	gdi.setData("PunchSync", 1);
	
  gdi.restoreNoUpdate("SpeakerList");
	gdi.setRestorePoint("SpeakerList");

	list<oSpeakerObject> speakerList;

  //For preliminary times
  updateComputerTime();
  oSpeakerObject so;

  speakerList.clear();
  
  if(classHasTeams(ClassId)) {
    oTeamList::iterator it=Teams.begin();
    while(it!=Teams.end()){
		  if(it->getClassId()==ClassId){		
	      it->fillSpeakerObject(leg, ControlId, so);
        if (so.owner)
			    speakerList.push_back(so);
		  }
		  ++it;
	  }	
  }
  else {
    oRunnerList::iterator it=Runners.begin();
    while(it!=Runners.end()){
		  if(it->getClassId()==ClassId){		
	      it->fillSpeakerObject(leg, ControlId, so);
        if (so.owner)
			    speakerList.push_back(so);
		  }
		  ++it;
	  }	
  }
	if(speakerList.empty()){
		gdi.addString("",0, "Inga deltagare");
		return;
	}

  list<oSpeakerObject>::iterator sit;
	for (sit=speakerList.begin(); sit != speakerList.end(); ++sit) {
		if(sit->status==StatusOK && sit->priority>=0)
			sit->priority=1;
		else if(sit->status > StatusOK  && sit->priority<=0)
			sit->priority=-1;
	}

	//Calculate place...
	calculateResults(speakerList);

	//Calculate preliminary times and sort by this and prio.
	speakerList.sort(CompareSpkSPList);	

  char bf[256];
  pClass pCls=oe->getClass(ClassId);

  if(!pCls)
    return;

  char bf2[64]="";
  if(pCls->getNumStages()>1)
    sprintf_s(bf2, "str. %d, ", leg+1);

  if(ControlId!=oPunch::PunchFinish) {
    pControl pCtrl=oe->getControl(ControlId, false);

    if(!pCtrl)
      return;

    string control=string(bf2)+lang.tl("kontroll")+" "+pCtrl->getName();
    sprintf_s(bf, "%s, %s", pCls->getName().c_str(), control.c_str());
  }
  else {
    string control=string(bf2)+lang.tl("ml");
    sprintf_s(bf, "%s, %s", pCls->getName().c_str(), control.c_str());
  }

  int y=gdi.getCY()+5;
	int x=30;

  gdi.addStringUT(y, x, 1, bf).setColor(92,92,128);
	int lh=gdi.getLineHeight();
 
  y+=lh*2;
	int LeaderTime=100000;

	//Calculate leader-time
	for(sit=speakerList.begin(); sit != speakerList.end(); ++sit){
		if(sit->status==StatusOK)	
      LeaderTime=min(LeaderTime, sit->runningTime);
	} 

  bool rendered=false;
	sit=speakerList.begin();
	while(sit != speakerList.end()){		
		if(sit->priority>0 || (sit->status==StatusOK && sit->priority>=0)){
			
      if(rendered==false) {
        gdi.addString("", y, x, boldSmall, "Resultat");
    	  y+=lh, rendered=true;
      }

      oSpeakerObject &so=*sit;
			oSpeakerObject *next_so=0;

			++sit;
			if(sit!=speakerList.end())
				next_so=&*sit;

			renderRowSpeakerList(gdi, so, next_so, x, y, LeaderTime, 1);
			y+=lh;
		}
		else ++sit;
	}
	
  rendered=false;
	sit=speakerList.begin();	
	while (sit != speakerList.end()) {		
		if (sit->status==StatusUnknown && sit->priority==0) {

      if(rendered==false) {
      	gdi.addString("", y+5, x, boldSmall, "Inkommande");
      	y+=lh+5, rendered=true;
      }

 			oSpeakerObject &so=*sit;
			oSpeakerObject *next_so=0;

			++sit;
			if(sit!=speakerList.end())
				next_so=&*sit;

			renderRowSpeakerList(gdi, so, next_so, x, y, LeaderTime, 2);
			y+=lh;
		}
		else ++sit;
	}

	sit=speakerList.begin();	
  rendered=false;
	while(sit != speakerList.end()){		
		if (!sit->isRendered) {
      if(rendered==false) {
        gdi.addString("", y+5, x, boldSmall, "vriga");
	      y+=lh+5, rendered=true;
      }
 			oSpeakerObject &so=*sit;
			oSpeakerObject *next_so=0;

			++sit;
			if(sit!=speakerList.end())
				next_so=&*sit;

			renderRowSpeakerList(gdi, so, next_so, x, y, LeaderTime, 3);
      y+=lh;
		}
		else ++sit;
	}
  gdi.refreshFast();
}

void oEvent::updateComputerTime()
{
	SYSTEMTIME st;	
	GetLocalTime(&st);

	ComputerTime=((24+2+st.wHour)*3600+st.wMinute*60+st.wSecond-ZeroTime)%(24*3600)-2*3600;
}

void oEvent::clearPrewarningSounds()
{
	oFreePunchList::reverse_iterator it;
	for (it=punches.rbegin(); it!=punches.rend(); ++it) 
		it->hasBeenPlayed=true;
}

void oEvent::tryPrewarningSounds(const string &basedir, int number)
{
	char wave[20];
	sprintf_s(wave, "%d.wav", number);

	string file=basedir+"\\"+wave;
	
	if(_access(file.c_str(), 0)==-1)
		MessageBox(0, "Error: hittar inte filen", file.c_str(), MB_OK);

	PlaySound(file.c_str(), 0, SND_SYNC|SND_FILENAME );			
}

void oEvent::playPrewarningSounds(const string &basedir, set<int> &controls)
{
	oFreePunchList::reverse_iterator it;
	for (it=punches.rbegin(); it!=punches.rend() && !it->hasBeenPlayed; ++it) {

    if (controls.count(it->Type)==1 || controls.empty()) {
		  pRunner r=getRunnerByCard(it->CardNo);

		  if(r){			
			  char wave[20];
			  sprintf_s(wave, "%d.wav", r->getStartNo());

			  string file=basedir+"\\"+r->getDI().getString("Nationality")+"\\"+wave;

			  if(_access(file.c_str(), 0)==-1)
				  file=basedir+"\\"+wave;
  			
			  PlaySound(file.c_str(), 0, SND_SYNC|SND_FILENAME );			
			  it->hasBeenPlayed=true;
		  }
    }
	}
}
