// xmlparser.h: interface for the xmlparser class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_XMLPARSER_H__87834E6D_6AB1_471C_8E1C_E65D67A4F98A__INCLUDED_)
#define AFX_XMLPARSER_H__87834E6D_6AB1_471C_8E1C_E65D67A4F98A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

#include <vector>
class xmlobject;

typedef vector<xmlobject> xmlList;
typedef const xmlobject * cxmlobject;

class xmlobject
{
protected:
  static string emptystring;
	string name;
	string value;
	bool isclosed;
	xmlobject *parent;
	xmlList objects;	
public:
	const string &GetName() const {return name;}
	const xmlobject *GetObject(const char *pname) const;

	int GetObjectInt(const char *pname) const
	{ 
		cxmlobject x=GetObject(pname);
		if(x) return atoi(x->value.c_str());
		else return 0;
	}
	
	const string &GetObjectString(const char *pname) const
	{ 
    cxmlobject x=GetObject(pname);
		if(x) return x->value;
		else return emptystring;
	}
		
	const xmlList *GetObjects() const {return &objects;}
	bool AddObject(xmlobject *xo);
	void Set(const string &pname, const string &pvalue);
	void Set(char *pname, char *pvalue);
	bool Is(const char *pname) const {return name==pname;}
	const string &Get() const {return value;}
	xmlobject();
	//xmlobject(xmlobh
	virtual ~xmlobject();
	friend class xmlparser;
};

class xmlparser  
{
protected:
	static char *ltrim(char *s);
	static const char *ltrim(const char *s);

	string tagStack[32];
	int tagStackPointer;
	ofstream fout;
	ifstream fin;

	int LineNumber;
	string ErrorMessage;
	xmlobject *xmlobj;
	string doctype;
	bool OnlyData;
	bool KillProcess;

	bool ParseXMLData(char *bf, xmlobject *xo);
	bool ParseTag(xmlobject *xo, string &tag);
public:
	bool Open(const char *file);
	bool Read(int MaxObj);
	xmlobject *CloneTakeBase();
	bool Failed(){return ErrorMessage.length()>0;}

	void GetOnlyData();
	const xmlobject * GetObject(const char *pname) const;
	const char *GetError();
	bool Read(const char *file);
	
	bool Write(const char *tag, const char *Property, 
             const string &Value);
  bool Write(const char *tag, const char *Property, 
             const string &PropValue, const string &Value);
	bool Write(const char *tag, const string &Value);
	bool Write(const char *tag, int);

	bool StartTag(const char *tag);	
	bool StartTag(const char *tag, const char *Property, 
                const string &Value);

	bool EndTag();

	
	bool CloseOut();
	bool OpenOutput(const char *file, const string &type);
	bool ReadXMLData(xmlobject *xo);
	xmlparser();
	virtual ~xmlparser();

};

#endif // !defined(AFX_XMLPARSER_H__87834E6D_6AB1_471C_8E1C_E65D67A4F98A__INCLUDED_)
